#!/usr/bin/env python3

import ROOT
from rich import print, prompt as rp
from time import perf_counter, gmtime


def varToStr(var):
    if isinstance(var, ROOT.RooCategory):
        states = {}
        for k, v in var.states():
            states[v] = k
        return (
            f"{var.GetName()}: {states[var.getCurrentIndex()]} {list(states.values())}"
        )
    else:
        bins = var.getBinning()
        return f"{var.GetName()}: {var.getValV()} ({var.getMin()}, {var.getMax()}) [|{bins.lowBound()}, {bins.highBound()}, {bins.numBins()}|]"


f = ROOT.TFile.Open("example_UsingPy_combined_meas_model.root")
w = f.Get("combined")
data = w.data("obsData")
n_evts = int(data.sumEntries() + 1)
pdf = w.pdf("simPdf")
params = pdf.getParameters(data)
observables = data.get()
weight_var = ROOT.RooRealVar("_weight_", "_weight_", 1, 0., 1e7)
observables.add(weight_var)
print("\n[blue bold]Observables [/blue bold]")
print(
    "\n".join([varToStr(o) for o in observables]),
)
data.Print()
print(f"\n[blue bold]Events per dataset: {n_evts}[/]")
print(f"[blue bold]Num entries: {data.numEntries()}[/]")
print(f"[blue bold]Expected events: {pdf.expectedEvents(observables)}[/]")
rp.Prompt.ask("Press Enter to continue")
print("\n")
print("[green bold]Running fit...[/]")
fit_start = perf_counter()
poi = params.selectByName("SigXsecOverSM").first()
poi.setVal(0.0)
poi.setConstant(True)
res = pdf.fitTo(
    data,
    ROOT.RooFit.Offset(True),
    ROOT.RooFit.BatchMode(True),
    ROOT.RooFit.Save(True),
)
print("FIT")
exit(0)
print("[blue bold]Parameters [/blue bold]")
print(
    "\n".join([varToStr(p) for p in params]),
)
print(f"Expected events: {pdf.expectedEvents(observables)}")
print(f"Took {perf_counter() - fit_start}s")
rp.Prompt.ask("Press Enter to continue")
print("[green bold]Generating toys...[/]")
w2 = ROOT.RooWorkspace("toys", "toys")
start = perf_counter()
spec = pdf.prepareMultiGen(
    observables,
    ROOT.RooFit.Name("toyData"),
    ROOT.RooFit.NumEvents(n_evts),
    ROOT.RooFit.Verbose(True),
    ROOT.RooFit.Extended(),
    ROOT.RooFit.AllBinned()
)
print(f"Took {perf_counter() - start} s to initialize")
for i in range(100):
    start = perf_counter()
    toy = pdf.generate(spec)
    toy.SetName(f"toyData_{i}")
    w2.Import(toy)
    print(f"Generated toy {i} with {int(toy.sumEntries())} events ({toy.numEntries()})")
    print(f"Took {perf_counter() - start} s")
print("[green bold]Saving workspace...[/]")
w2.writeToFile("toys.root")
